from typing import Dict
try:
    # When running from this package
    from location_local_constants import LocationLocalConstants
except Exception:
    # When importing this module from another package
    from location_local.location_local_constants import LocationLocalConstants
from dotenv import load_dotenv
load_dotenv()
from circles_local_database_python.generic_crud import GenericCRUD  # noqa: E402
from circles_local_database_python.connector import Connector   # noqa: E402
from logger_local.Logger import Logger  # noqa: E402
from language_local_python_package.src.language_enum_class import LanguageCode  # noqa: E402


logger = Logger.create_logger(object=LocationLocalConstants.OBJECT_FOR_LOGGER_CODE)


class Region(GenericCRUD):
    def __init__(self):
        logger.start()

        self.connector = Connector.connect("location")
        self.cursor = self.connector.cursor()

        logger.end()

    def insert_region(
            self, coordinate: Dict[str, float],
            region: str, lang_code: LanguageCode = 'en', title_approved: bool = False):
        logger.start(object={'coordinate': coordinate, 'region': region, 'lang_code': lang_code, 'title_approved': title_approved})
        query_insert = "INSERT IGNORE INTO region_table (coordinate) VALUES (POINT(%s, %s))"
        self.cursor.execute(
            query_insert, (coordinate["latitude"], coordinate["longitude"]))
        region_id = self.cursor.lastrowid()
        query_insert_ml = "INSERT IGNORE INTO region_ml_table (region_id, lang_code, title, title_approved)" \
            " VALUES (%s, %s, %s, %s)"
        self.cursor.execute(query_insert_ml, (region_id,
                            lang_code, region, title_approved))
        self.connector.commit()
        logger.end(object={'region_id': region_id})
        return region_id

    def get_region_id(self, title):
        logger.start(object={'title': title})

        query_get = "SELECT region_id FROM region_ml_view WHERE title = %s"
        self.cursor.execute(query_get, (title,))
        region_id = self.cursor.fetchone()
        if region_id is not None:
            region_id = region_id[0]
        self.cursor.fetchall()

        logger.end(object={'region_id': region_id})
        return region_id



from logger_local.LoggerComponentEnum import LoggerComponentEnum


class LocationLocalConstants:
    LOCATION_LOCAL_PYTHON_PACKAGE_COMPONENT_ID = 113
    LOCATION_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME = 'location local python package'
    OBJECT_FOR_LOGGER_CODE = {
        'component_id': LOCATION_LOCAL_PYTHON_PACKAGE_COMPONENT_ID,
        'component_name': LOCATION_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME,
        'component_category': LoggerComponentEnum.ComponentCategory.Code.value,
        'developer_email': 'tal.g@circ.zone'
    }

    OBJECT_FOR_LOGGER_TEST = {
        'component_id': LOCATION_LOCAL_PYTHON_PACKAGE_COMPONENT_ID,
        'component_name': LOCATION_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME,
        'component_category': LoggerComponentEnum.ComponentCategory.Unit_Test.value,
        'developer_email': 'tal.g@circ.zone'
    }

    UNKNOWN_LOCATION_ID = 37522



import os
from typing import Dict
from opencage.geocoder import OpenCageGeocode
try:
    # When running from this package
    from location_local_constants import LocationLocalConstants
except Exception:
    # When importing this module from another package
    from location_local.location_local_constants import LocationLocalConstants
from dotenv import load_dotenv
load_dotenv()
from circles_local_database_python.generic_crud import GenericCRUD  # noqa: E402
from circles_local_database_python.connector import Connector  # noqa: E402
from logger_local.Logger import Logger  # noqa: E402
from language_local_python_package.src.language_enum_class import LanguageCode  # noqa: E402
api_key = os.getenv("OPENCAGE_KEY")

logger = Logger.create_logger(object=LocationLocalConstants.OBJECT_FOR_LOGGER_CODE)


class Country(GenericCRUD):
    def __init__(self):
        logger.start()

        self.connector = Connector.connect("location")
        self.cursor = self.connector.cursor()

        logger.end()

    def insert_country(self, coordinate: Dict[str, float],
                       country: str, lang_code: LanguageCode = 'en', title_approved: bool = False,
                       new_country_data: Dict[str, any] = None):
        logger.start(object={'coordinate': coordinate, 'country': country, 'lang_code': lang_code, 'title_approved': title_approved, 'new_country_data': new_country_data})
        try:
            query_insert = "INSERT IGNORE INTO country_table (coordinate, iso, `name`, nicename, iso3, numcode, phonecode) VALUES (POINT(%s, %s), %s, %s, %s, %s, %s, %s)"
            self.cursor.execute(
                query_insert,
                (coordinate["latitude"],
                 coordinate["longitude"],
                 new_country_data["iso"],
                 new_country_data["name"],
                 new_country_data["nicename"],
                 new_country_data["iso3"],
                 new_country_data["numcode"],
                 new_country_data["phonecode"]))
        except Exception as e:
            logger.exception("error in insert country", e)
            logger.end()
            raise e
        try:
            country_id = self.cursor.lastrowid()
            query_insert_ml = "INSERT IGNORE INTO country_ml_table (country_id, lang_code, title," \
                " title_approved) VALUES (%s, %s, %s, %s)"
            self.cursor.execute(query_insert_ml, (country_id,
                                                  lang_code, country, title_approved))
            self.connector.commit()
        except Exception as e:
            logger.exception("error in insert country", e)
            logger.end()
            raise e
        logger.end(object={'country_id': country_id})
        return country_id

    def get_country_id(self, name):
        logger.start(object={'name': name})

        query_get = "SELECT country_id FROM country_ml_view WHERE `name` = %s"
        self.cursor.execute(query_get, (name,))
        country_id = self.cursor.fetchone()
        if country_id is not None:
            country_id = country_id[0]
        self.cursor.fetchall()

        logger.end(object={'country_id': country_id})
        return country_id

    @staticmethod
    def get_country_name(location):
        # Create a geocoder instance
        logger.start(object={'location': location})

        # Define the city or state
        geocoder = OpenCageGeocode(api_key)

        # Use geocoding to get the location details
        results = geocoder.geocode(location)

        if results and len(results) > 0:
            first_result = results[0]
            components = first_result['components']

            # Extract the country from components
            country_name = components.get('country', '')
            if not country_name:
                # If country is not found, check for country_code as an alternative
                country_name = components.get('country_code', '')
        else:
            country_name = None
            logger.error("country didnt  found for %s." % location)
        logger.end(object={'country_name': country_name})
        return country_name




import setuptools

PACKAGE_NAME = "location-local"
package_dir = PACKAGE_NAME.replace("-", "_")

setuptools.setup(
    name=PACKAGE_NAME,
    version='0.0.42',  # https://pypi.org/project/location-local/
    author="Circles",
    author_email="info@circles.life",
    url=f"https://github.com/circles-zone/{PACKAGE_NAME}-python-package",
    packages=[package_dir],
    package_dir={package_dir: f'{package_dir}/src'},
    package_data={package_dir: ['*.py']},
    description="Location Locatal PyPI Package",
    long_description="This is a package for sharing common OpenCage function used in different repositories",
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Other/Proprietary License",
        "Operating System :: OS Independent",
    ],
    install_requires=["mysqlclient",
                    "mysql-connector>=2.2.9",
                    "pytest>=7.4.1",
                    "database-without-orm-local>=0.0.79",
                    "logger-local>=0.0.59",
                    "language-local>=0.0.4",
                    "python-dotenv>=1.0.0",
                    "logzio-python-handler>=4.1.0",
                    "opencage>=2.3.0"]
)



from typing import Dict
try:
    # When running from this package
    from location_local_constants import LocationLocalConstants
    from neighborhood import Neighborhood
    from county import County
    from region import Region
    from state import State
    from country import Country
except Exception:
    # When importing this module from another package
    from location_local.location_local_constants import LocationLocalConstants
    from location_local.neighborhood import Neighborhood
    from location_local.county import County
    from location_local.region import Region
    from location_local.state import State
    from location_local.country import Country
from dotenv import load_dotenv
load_dotenv()
from circles_local_database_python.generic_crud import GenericCRUD   # noqa: E402
from circles_local_database_python.connector import Connector   # noqa: E402
from logger_local.Logger import Logger  # noqa: E402
from user_context_remote.user_context import UserContext  # noqa: E402
from language_local_python_package.src.language_enum_class import LanguageCode  # noqa: E402


logger = Logger.create_logger(object=LocationLocalConstants.OBJECT_FOR_LOGGER_CODE)

user_context = UserContext().login()


class LocationLocal(GenericCRUD):
    def __init__(self):
        logger.start()

        self.connector = Connector.connect("location")
        self.cursor = self.connector.cursor()

        logger.end()

    def get_location_table_ids(
            self, neighborhood: str, county: str, region: str, state: str, country: str) -> tuple(
            [int, int, int, int, int]):
        logger.start(object={"neighborhood": neighborhood, "county": county, "region": region, "state": state, "country": country})

        neighborhood_id, county_id, region_id, state_id, country_id = None, None, None, None, None
        neighborhood_object = Neighborhood()
        neighborhood_id = neighborhood_object.get_neighborhood_id(neighborhood)
        county_object = County()
        county_id = county_object.get_county_id(county)
        region_object = Region()
        region_id = region_object.get_region_id(region)
        state_object = State()
        state_id = state_object.get_state_id(state)
        country_object = Country()
        country_id = country_object.get_country_id(country)

        logger.end(object={"neighborhood_id": neighborhood_id, "county_id": county_id, "region_id": region_id,
                    "state_id": state_id, "country_id": country_id})
        return (neighborhood_id, county_id, region_id, state_id, country_id)

    def insert(
            self, location_data: Dict[str, any],
            lang_code: LanguageCode = "en", is_approved: bool = False, new_country_data: Dict[str, any] = None):
        logger.start(object={"location_data": location_data, "lang_code": lang_code, "is_approved": is_approved, "new_country_data": new_country_data})
        (neighborhood_id, county_id, region_id, state_id, country_id) = self._check_details_and_insert_if_not_exist(
            location_data["coordinate"],
            (location_data["neighborhood"],
             location_data["county"],
             location_data["region"],
             location_data["state"],
             location_data["country"]),
            lang_code, is_approved, new_country_data)

        query_insert = "INSERT INTO location_table (coordinate, address_local_language, address_english," \
            " neighborhood_id, county_id, region_id, state_id, country_id, postal_code, plus_code, is_approved)" \
            " VALUES (POINT(%s, %s), %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)"
        self.cursor.execute(
            query_insert,
            (location_data["coordinate"]["latitude"],
             location_data["coordinate"]["longitude"],
             location_data["address_local_language"],
             location_data["address_english"],
             neighborhood_id, county_id, region_id, state_id, country_id, location_data["postal_code"],
             location_data["plus_code"],
             is_approved))
        self.connector.commit()

        location_id = self.cursor.lastrowid()

        logger.end(object={"location_id": location_id})
        return location_id

    def update(self, location_id: int, location_data: Dict[str, any],
               lang_code: LanguageCode = "en", is_approved: bool = False):
        logger.start(object={"location_id": location_id, "location_data": location_data, "lang_code": lang_code, "is_approved": is_approved})
        (neighborhood_id, county_id, region_id, state_id, country_id) = self._check_details_and_insert_if_not_exist(
            location_data["coordinate"],
            (location_data["neighborhood"],
             location_data["county"],
             location_data["region"],
             location_data["state"],
             location_data["country"]),
            lang_code, is_approved)

        query_update = "UPDATE location_table SET coordinate = POINT(%s, %s), address_local_language = %s, " \
            "address_english = %s, neighborhood_id = %s, county_id = %s, region_id = %s, state_id = %s," \
            " country_id = %s, postal_code = %s, plus_code = %s, is_approved = %s WHERE location_id = %s"
        self.cursor.execute(
            query_update,
            (location_data["coordinate"]["latitude"],
             location_data["coordinate"]["longitude"],
             location_data["address_local_language"],
             location_data["address_english"],
             neighborhood_id, county_id, region_id, state_id, country_id, location_data["postal_code"],
             location_data["plus_code"],
             is_approved,
             location_id))
        self.connector.commit()
        logger.end(object={"location_id": location_id})

    def read(self, location_id: int) -> tuple([Dict[str, float], str, str, str, str, str, str, str, str, str]):
        logger.start(object={"location_id": location_id})

        query_get = '''SELECT ST_X(l.coordinate), ST_Y(l.coordinate), l.address_local_language, l.address_english, n.title, c.title, 
                            r.title, s.state_name, co.name, l.postal_code, l.plus_code 
                            FROM location_view l 
                            JOIN neighborhood_ml_view n ON n.neighborhood_id = l.neighborhood_id
                            JOIN county_ml_view c ON c.county_id = l.county_id
                            JOIN region_ml_view r ON r.region_id = l.region_id
                            JOIN state_ml_view s ON s.state_id = l.state_id 
                            JOIN country_table co ON co.country_id = l.country_id 
                            WHERE l.location_id = %s'''
        self.cursor.execute(query_get, (location_id,))
        result = None
        coordinate = None
        result = self.cursor.fetchone()
        if result is None:
            logger.end(object={"result": result})
            return None

        (latitude, longitude, address_local_language, address_english, neighborhood,
         county, region, state, country, postal_code, plus_code) = result
        coordinate = {"latitude": latitude, "longitude": longitude}

        logger.end(object={"address_local_language": address_local_language, "address_english": address_english,
                    "neighborhood": neighborhood, "county": county, "region": region, "state": state,
                    "country": country, "postal_code": postal_code, "plus_code": plus_code})
        return (coordinate, address_local_language, address_english, neighborhood, county, region,
                state, country, postal_code, plus_code)

    def delete(self, location_id: int):
        logger.start(object={"location_id": location_id})

        query_update = "UPDATE location_table SET end_timestamp = NOW() WHERE location_id = %s"
        self.cursor.execute(query_update, (location_id,))

        self.connector.commit()
        logger.end()

    def _check_details_and_insert_if_not_exist(
            self, coordinate: Dict[str, float],
            location_details: tuple([str, str, str, str, str]),
            lang_code: LanguageCode = 'en', is_approved=False, new_country_data: Dict[str, any] = None) -> tuple(
            [int, int, int, int, int]):
        logger.start()
        (neighborhood, county, region, state, country) = location_details
        ids = self.get_location_table_ids(
            neighborhood, county, region, state, country)
        if ids is None:
            return None
        (neighborhood_id, county_id, region_id, state_id, country_id) = ids
        if neighborhood_id is None:
            neighborhood_object = Neighborhood()
            neighborhood_id = neighborhood_object.insert_neighborhood(
                coordinate, neighborhood, lang_code, )
        if county_id is None:
            county_object = County()
            county_id = county_object.insert_county(coordinate, county, lang_code, is_approved)
        if region_id is None:
            region_object = Region()
            region_id = region_object.insert_region(coordinate, region, lang_code, is_approved)
        if state_id is None:
            state_object = State()
            state_id = state_object.insert_state(coordinate, state, lang_code, is_approved)
        if country_id is None:
            country_object = Country()
            country_id = country_object.insert_country(coordinate, country, lang_code, is_approved, new_country_data)
        logger.end(object={"neighborhood_id": neighborhood_id, "county_id": county_id, "region_id": region_id, "state_id": state_id, "country_id": country_id})
        return (neighborhood_id, county_id, region_id, state_id, country_id)
